using System;
using System.Xml;
using System.Collections;
using System.Xml.Serialization;
using Microsoft.Win32;

namespace SigKnockClient
{
	/// <summary>
	/// Summary description for Einstellungen.
	/// </summary>
	public class Einstellungen 
	{
		[XmlElement("gpgpath")] public String gpg_path;
		[XmlElement("pubkeyring")] public String pubkeyring;
		[XmlElement("seckeyring")] public String seckeyring; 
		private ArrayList listsessions;

		public Einstellungen()
		{
			// some defaults ...
			RegistryKey regsigknock=Registry.CurrentUser.OpenSubKey("Software").OpenSubKey("SigKnock");
			if(regsigknock==null)
			{
				// SigKnock in der Registry nicht auffindbar - Fehlerhafte Installation?
				gpg_path="(can't find installation directory registry entry for SigKnock - installation damaged?)";
			}
			else
			{
				gpg_path=regsigknock.GetValue("")+"\\gpg.exe";
			}
            			
			pubkeyring=Environment.GetFolderPath(Environment.SpecialFolder.Personal)+"\\GPG\\pubring.gpg";
			seckeyring=Environment.GetFolderPath(Environment.SpecialFolder.Personal)+"\\GPG\\secring.gpg";
			listsessions=new ArrayList();
		}

		public Einstellungen(Einstellungen e)
		{
			this.gpg_path=e.gpg_path;
			this.pubkeyring=e.pubkeyring;
			this.seckeyring=e.seckeyring;
			this.sessions=e.sessions;
		}

		[XmlElement("session")]
		public SessionContainer[] sessions
		{
			get
			{
				SessionContainer[] sessions=new SessionContainer[listsessions.Count];
				listsessions.CopyTo(sessions);
				return sessions;
			}

			set
			{
				if(value==null)
					return;
				SessionContainer[] sessions=(SessionContainer[])value;
				listsessions.Clear();
				foreach(SessionContainer s in sessions)
				{
					listsessions.Add(s);
				}
			}
		}

		public void AddSession(SessionContainer cnt)
		{
			listsessions.Add(cnt);
		}

		public void RemoveSession(SessionContainer cnt)
		{
			listsessions.Remove(cnt);
		}

		public void ReplaceSession(SessionContainer cntold, SessionContainer cnt)
		{
			int idx=listsessions.IndexOf(cntold);
			listsessions.Remove(cntold);
			listsessions.Insert(idx, cnt);
		}
	}
}
