using System;
using System.IO;
using System.Collections;
using System.Diagnostics;
using System.Windows.Forms;

namespace SigKnockOpener
{
	/// <summary>
	/// Summary description for Win32SignHelper.
	/// </summary>
	public class Win32SignHelper : SigKnockSignHelper
	{
		private String pubkeyring, seckeyring, pathToGpg;

		public Win32SignHelper(String path_to_gpg, String pubkeyring, String seckeyring)
		{
			this.pubkeyring=pubkeyring;
			this.seckeyring=seckeyring;
			this.pathToGpg=path_to_gpg;
		}

		private void signFile(String challenge_file, 
			String signature_file, String UID, String passphrase)
		{
			Process p=new Process();

			ProcessStartInfo pi=new ProcessStartInfo(this.pathToGpg);
			String[] parameter=new String[5];
			parameter[0]=this.pubkeyring;
			parameter[1]=this.seckeyring;
			parameter[2]=signature_file;
			parameter[3]=UID;
			parameter[4]=challenge_file;
			pi.Arguments=String.Format("--no-default-keyring --keyring \"{0}\" --secret-keyring \"{1}\" "+
				"--detach-sign --no-random-seed-file --no-options --output \"{2}\" -u 0x{3} \"{4}\"", parameter);
			pi.UseShellExecute = true;

			p.StartInfo = pi;
			Console.Out.WriteLine(pi.FileName+" "+pi.Arguments);

			File.Delete(signature_file);
			p.Start();

			// this shouldn't take longer than 90 seconds
			p.WaitForExit(90*1000);
			if(!p.HasExited)
				p.Kill();
		}

		public override byte[] sign(byte[] challenge, String UID)
		{
			String passphrase="";
			
			String challenge_file=Path.GetTempFileName();
			String signature_file=Path.GetTempFileName();
			
			BinaryWriter cbw=new BinaryWriter(new FileStream(challenge_file, FileMode.OpenOrCreate, FileAccess.Write));
			cbw.Write(challenge);
			cbw.Close();

			/*Win32PasswordDialog dlg=new Win32PasswordDialog();
			dlg.ShowDialog();
			passphrase=dlg.Password;*/
			
			signFile(challenge_file, signature_file, UID, passphrase);

			BinaryReader sbr=new BinaryReader(new FileStream(signature_file, FileMode.Open, FileAccess.Read));

			byte[] signature=sbr.ReadBytes(10000);

			sbr.Close();
			File.Delete(challenge_file);
			File.Delete(signature_file);

			return signature;
		}

		public override ArrayList getKeyIDs()
		{
			ArrayList rval=new ArrayList();
			Process p=new Process();
			StreamWriter sw;
			StreamReader sr, err;

			ProcessStartInfo pi=new ProcessStartInfo(this.pathToGpg);
			pi.Arguments=String.Format("--no-default-keyring --with-colons --no-options --keyring \"{0}\" --secret-keyring \"{1}\" --list-secret-keys",
				this.pubkeyring, this.seckeyring);
			pi.UseShellExecute = false;
			pi.RedirectStandardInput = true;
			pi.RedirectStandardOutput = true;
			pi.RedirectStandardError = true;
			pi.CreateNoWindow = true;
			p.StartInfo = pi;

			p.Start();
			sw = p.StandardInput;
			sr = p.StandardOutput;
			err = p.StandardError;
			sw.AutoFlush = true;

			String erg=sr.ReadToEnd();
			foreach(String line in erg.Split(new char[] {'\n','\r'}))
			{
				try
				{
					if(line.StartsWith("sec"))
					{
						String[] cols=line.Split(':');
						rval.Add(new UserIDKeyIDTupel(cols[9],cols[4]));
					}
				}
				catch(IndexOutOfRangeException)
				{
					// ignored
				}
			}

			sr.Close();
			sw.Close();
			err.Close();
            
			return rval;
		}
	}
}
